from abc import ABC, abstractmethod
from typing import Any, Dict, List, Optional, Type


class LLM(ABC):
    """LLMの基底クラス"""

    def __init__(self, model_name: str, llm_key: Optional[str] = None):
        self.model_name = model_name
        self.llm_key = llm_key

    @abstractmethod
    def call(self, messages: List[Dict[str, str]]) -> str:
        """LLMを呼び出して応答を取得する"""
        pass

    @classmethod
    def configure(cls, model_name: str, llm_key: Optional[str] = None) -> "LLM":
        """LLMインスタンスを設定する"""
        return cls(model_name=model_name, llm_key=llm_key)



import os
from unittest.mock import patch

import pytest
from pydantic import BaseModel

from dariko import set_config


class Person(BaseModel):
    name: str
    age: int
    dummy: bool


@pytest.fixture(autouse=True)
def set_api_key():
    """テスト用のAPIキーを設定する"""
    os.environ["DARIKO_API_KEY"] = "test_key"
    set_config(model="gpt-4o-mini", llm_key="test_key")


def mock_gpt_response(*args, **kwargs):
    """GPTモデルのモックレスポンス"""
    class MockResponse:
        def __init__(self):
            self.status_code = 200
            self._json = {
                "choices": [
                    {
                        "message": {
                            "content": '{"name": "test", "age": 20, "dummy": true}'
                        }
                    }
                ]
            }

        def json(self):
            return self._json

        def raise_for_status(self):
            pass

    return MockResponse()


def mock_gemma_response(*args, **kwargs):
    """Gemmaモデルのモックレスポンス"""
    return '{"name": "test", "age": 20, "dummy": true}'


def mock_claude_response(*args, **kwargs):
    """Claudeモデルのモックレスポンス"""
    class MockResponse:
        def __init__(self):
            self.status_code = 200
            self._json = {
                "content": [
                    {
                        "text": '{"name": "test", "age": 20, "dummy": true}'
                    }
                ]
            }

        def json(self):
            return self._json

        def raise_for_status(self):
            pass

    return MockResponse()


def mock_invalid_response(*args, **kwargs):
    """無効なレスポンスのモック"""
    class MockResponse:
        def __init__(self):
            self.status_code = 200
            self._json = {"choices": [{"message": {"content": '{"invalid": "response"}'}}]}

        def json(self):
            return self._json

    return MockResponse() 



from unittest.mock import patch

from dariko import ask, set_config
from tests.conftest import Person, mock_claude_response


@patch("dariko.models.claude.requests.post", side_effect=mock_claude_response)
def test_configure_claude(mock_post):
    """Claudeモデルの設定テスト"""
    # AnthropicのAPIキーを設定
    set_config(model="claude-3-opus-20240229", llm_key="test_anthropic_key")
    result: Person = ask("test", output_model=Person)
    assert result.dummy is True 



# llm.py
from __future__ import annotations

import inspect
import json
from typing import Any, List, Type, Dict

from pydantic import BaseModel, TypeAdapter
from pydantic import ValidationError as _PydanticValidationError

from .config import get_llm_key, get_model
from .exceptions import ValidationError
from .model_utils import get_pydantic_model, infer_output_model
from .models.llm import LLM
from .models.gpt import GPT
from .models.gemma import Gemma
from .models.claude import Claude

# モデル名とLLMクラスのマッピング
MODEL_MAPPING: Dict[str, Type[LLM]] = {
    "gpt": GPT,
    "gemma": Gemma,
    "claude": Claude,
}

# ─────────────────────────────────────────────────────────────
# 内部ユーティリティ
# ─────────────────────────────────────────────────────────────


def _resolve_model(output_model: Type[Any] | None) -> Type[BaseModel]:
    """
    output_model が None の場合は呼び出しフレームから推論し、
    最終的に Pydantic Model 型を返す。
    """
    if output_model is None:
        caller_frame = inspect.currentframe().f_back
        model = infer_output_model(caller_frame)
        if model is None:
            raise TypeError("型アノテーションが取得できませんでした。output_model を指定してください。")
    else:
        model = output_model
    return get_pydantic_model(model)  # 型チェックも兼ねる


def _get_llm_instance() -> LLM:
    """
    設定に基づいて適切なLLMインスタンスを返す
    """
    model_name = get_model()
    llm_key = get_llm_key()

    # モデル名からLLMクラスを特定
    for prefix, llm_class in MODEL_MAPPING.items():
        if prefix in model_name.lower():
            return llm_class.configure(model_name=model_name, llm_key=llm_key)

    raise ValueError(f"Unsupported model: {model_name}")


def _post_to_llm(messages: list[dict[str, str]]) -> str:
    """
    LLMを呼び出して content 文字列を返す。
    """
    llm = _get_llm_instance()
    return llm.call(messages)


def _parse_and_validate(raw_json: str, pyd_model: Type[BaseModel], *, llm_key: str) -> BaseModel:
    """
    LLM 出力(JSON文字列)を parse & Pydantic 検証。
    成功すれば Pydantic モデルのインスタンスを返す。
    """
    try:
        data = json.loads(raw_json)
        return TypeAdapter(pyd_model).validate_python(data)
    except json.JSONDecodeError as e:
        raise ValidationError(
            _PydanticValidationError.from_exception_data(
                "JSONDecodeError",
                [{"loc": (), "msg": f"LLMの出力がJSONとして解析できませんでした: {e}", "type": "value_error"}],
            )
        ) from None
    except _PydanticValidationError as e:
        raise ValidationError(e) from None


# ─────────────────────────────────────────────────────────────
# パブリック API
# ─────────────────────────────────────────────────────────────
def ask(prompt: str, *, output_model: Type[Any] | None = None) -> Any:
    """
    単一プロンプトを実行し、Pydantic 検証済みオブジェクトを返す。
    """
    pyd_model = _resolve_model(output_model)
    llm_key = get_llm_key()

    raw = _post_to_llm(
        [
            {"role": "system", "content": f"{pyd_model.model_json_schema()}"},
            {"role": "user", "content": prompt},
        ]
    )
    return _parse_and_validate(raw, pyd_model, llm_key=llm_key)


def ask_batch(prompts: List[str], *, output_model: Type[Any] | None = None) -> List[Any]:
    """
    複数プロンプトをバッチ処理し、検証済みオブジェクトをリストで返す。
    """
    pyd_model = _resolve_model(output_model)
    llm_key = get_llm_key()

    results: list[Any] = []
    for p in prompts:
        raw = _post_to_llm(
            [
                {"role": "system", "content": f"{pyd_model.model_json_schema()}"},
                {"role": "user", "content": p},
            ]
        )
        results.append(_parse_and_validate(raw, pyd_model, llm_key=llm_key))
    return results



from unittest.mock import patch

import pytest

from dariko import ask, set_config, ValidationError
from tests.conftest import Person, mock_invalid_response


def test_validation_error():
    """バリデーションエラーのテスト"""
    set_config(model="gpt-4o-mini", llm_key="test_key")

    with patch("dariko.models.gpt.requests.post", side_effect=mock_invalid_response):
        with pytest.raises(ValidationError):
            ask("invalid", output_model=Person)


def test_unsupported_model():
    """未サポートモデルのテスト"""
    with pytest.raises(ValueError, match="Unsupported model"):
        set_config(model="unsupported-model", llm_key="test_key")
        ask("test", output_model=Person) 



from __future__ import annotations

import ast
import inspect
import logging
import os
from pathlib import Path
from typing import Any, Type, get_type_hints

from pydantic import BaseModel

# ロガーの設定
logger = logging.getLogger(__name__)


# ─────────────────────────────────────────────────────────────
# 内部ユーティリティ
# ─────────────────────────────────────────────────────────────
def _validate(model: Any) -> Type[BaseModel] | None:
    """Pydantic Model かどうかを判定し、list[T] にも対応する。BaseModelそのものは除外。"""
    origin = getattr(model, "__origin__", None)
    if origin is list:  # list[T] -> T を取り出す
        model = model.__args__[0]
    # BaseModelそのものは除外
    if model is BaseModel:
        return None
    return model if inspect.isclass(model) and issubclass(model, BaseModel) else None


def _model_from_ast(frame) -> Type[BaseModel] | None:
    """直前行以前の AnnAssign または Assign+type_comment から型を推定。"""
    # 呼び出し元のフレームを取得
    caller_frame = frame.f_back
    if caller_frame is None:
        logger.debug("No caller frame found")
        return None

    # 呼び出し元のファイルをパース
    try:
        file_path = Path(caller_frame.f_code.co_filename)
        logger.debug(f"Parsing file: {file_path}")
        tree = ast.parse(file_path.read_text(encoding="utf-8"))
    except Exception as e:
        logger.debug(f"Failed to parse file: {e}")
        return None

    caller_line = caller_frame.f_lineno
    logger.debug(f"Caller line: {caller_line}")

    # 関数の戻り値の型アノテーションを探す
    for node in ast.walk(tree):
        if isinstance(node, ast.FunctionDef):
            if node.returns is not None:
                try:
                    ann_type_str = ast.unparse(node.returns)
                    logger.debug(f"Found function return type: {ann_type_str}")
                    ann = eval(ann_type_str, caller_frame.f_globals, caller_frame.f_locals)
                    if model := _validate(ann):
                        return model
                except Exception as e:
                    logger.debug(f"Failed to evaluate function return type: {e}")

    # 変数の型アノテーションを探す
    for node in ast.walk(tree):
        node_line = getattr(node, "lineno", 0)
        if node_line > caller_line:
            continue

        # AnnAssign: result: Person = ...
        if isinstance(node, ast.AnnAssign):
            logger.debug(f"Found AnnAssign at line {node_line}")
            try:
                ann_type_str = ast.unparse(node.annotation)
                logger.debug(f"Type annotation string: {ann_type_str}")
                ann = eval(ann_type_str, caller_frame.f_globals, caller_frame.f_locals)
                if model := _validate(ann):
                    return model
            except Exception as e:
                logger.debug(f"Failed to evaluate annotation: {e}")

        # Assign + type_comment: result = ...  # type: Person
        elif isinstance(node, ast.Assign):
            if hasattr(node, "type_comment") and node.type_comment:
                logger.debug(f"Found Assign with type_comment at line {node_line}")
                try:
                    ann = eval(node.type_comment, caller_frame.f_globals, caller_frame.f_locals)
                    if model := _validate(ann):
                        return model
                except Exception as e:
                    logger.debug(f"Failed to evaluate type comment: {e}")

    logger.debug("No suitable type annotation found")
    return None


# ─────────────────────────────────────────────────────────────
# パブリック API
# ─────────────────────────────────────────────────────────────
def infer_output_model(frame=None) -> Type[BaseModel] | None:
    """
    実行中フレームから Pydantic モデル型を推定するユーティリティ。
    優先順位:
        1. 呼び出し元関数の return 型ヒント（関数オブジェクト or AST）
        2. 現フレームのローカル変数アノテーション
        3. AST 解析による推定
    """
    import ast
    import inspect

    # ------------------------------------------------------------
    # ❶ 呼び出し側から frame が渡された場合はそれを最優先
    # ------------------------------------------------------------
    if frame is not None:
        user_frame = frame
    else:
        # ❷ さもなくば「site-packages 内の dariko」を除いた最初のフレーム
        stack = inspect.stack()
        for s in stack:
            path = os.path.abspath(s.filename)
            # site-packages or editable install の dariko パッケージだけを除外
            if path.endswith("dariko/llm.py") or "/site-packages/dariko/" in path:
                continue
            user_frame = s.frame
            break
        else:
            user_frame = stack[-1].frame

    frame = user_frame
    logger.debug(f"Current frame: {frame.f_code.co_name} at line {frame.f_lineno} " f"in {frame.f_code.co_filename}")

    # 0) 現在実行中の関数 get_person_info() の return 型を調べる
    if frame.f_code.co_name != "<module>":
        try:
            # 関数オブジェクトを frame から解決
            func_obj = frame.f_locals.get(frame.f_code.co_name) or frame.f_globals.get(frame.f_code.co_name)
            if func_obj:
                return_type = get_type_hints(func_obj).get("return")
                logger.debug(f"Current func return type: {return_type}")
                if model := _validate(return_type):
                    return model

            # --- AST fallback ---
            file_path = Path(frame.f_code.co_filename)
            with file_path.open(encoding="utf-8") as f:
                tree = ast.parse(f.read())
            for node in ast.walk(tree):
                if isinstance(node, ast.FunctionDef) and node.name == frame.f_code.co_name:
                    if node.returns is not None:
                        ann_type_str = ast.unparse(node.returns)
                        logger.debug(f"AST: current func return annotation: {ann_type_str}")
                        try:
                            ann = eval(ann_type_str, frame.f_globals, frame.f_locals)
                            if model := _validate(ann):
                                return model
                        except Exception as e:
                            logger.debug(f"Failed to evaluate annotation: {e}")
                            continue
        except Exception as e:
            logger.debug(f"Failed to inspect current function: {e}")

    # 1) AST 解析による推定（変数の型アノテーションを含む）
    return _model_from_ast(frame)


def get_pydantic_model(model: Type[Any]) -> Type[BaseModel]:
    """
    与えられた型が Pydantic モデル（あるいは list[T] 形式で T が Pydantic
    モデル）かどうかを確認し、適切でなければ TypeError を投げる。
    """
    validated = _validate(model)
    if validated is None:
        raise TypeError("output_model must be a Pydantic model (or list[Model]).")
    return validated



import os
from pydantic import BaseModel

from dariko import ask, ask_batch, set_config

# 環境変数からAPIキーを取得
llm_key = os.environ.get("DARIKO_API_KEY")
set_config(model="gpt-4o-mini", llm_key=llm_key)


# 出力モデル定義
class Person(BaseModel):
    name: str
    age: int
    dummy: bool


prompt = "以下の形式でJSONを返してください:\n" + '{"name": "山田太郎", "age": 25, "dummy": false}'
result: Person = ask(prompt, output_model=Person)
print(result)

# バッチ処理
prompts = [
    "以下の形式でJSONを返してください:\n" + '{"name": "山田太郎", "age": 25, "dummy": false}',
    "以下の形式でJSONを返してください:\n" + '{"name": "佐藤花子", "age": 30, "dummy": true}',
]

results = ask_batch(prompts, output_model=Person)

# 結果表示
for i, result in enumerate(results, 1):
    print(f"\nPerson {i}:")
    print(f"Name: {result.name}")
    print(f"Age: {result.age}")
    print(f"Dummy: {result.dummy}")



# file generated by setuptools-scm
# don't change, don't track in version control

from dariko.config import set_config
from dariko.driver import ask, ask_batch, ValidationError

__version__ = "0.2.2"
__version_tuple__ = (0, 2, 2)
version = __version__
version_tuple = __version_tuple__

__all__ = [
    "set_config",
    "ask",
    "ask_batch",
    "ValidationError",
    "__version__",
    "__version_tuple__",
    "version",
    "version_tuple",
]

TYPE_CHECKING = False
if TYPE_CHECKING:
    from typing import Union

    VERSION_TUPLE = tuple[Union[int, str], ...]



import os
from unittest.mock import patch

import pytest

from dariko import ask, set_config
from tests.conftest import Person, mock_gpt_response


@patch("dariko.models.gpt.requests.post", side_effect=mock_gpt_response)
def test_configure_gpt(mock_post):
    """GPTモデルの設定テスト"""
    # 環境変数から設定
    os.environ["DARIKO_API_KEY"] = "test_key"
    set_config(model="gpt-4o-mini", llm_key="test_key")
    result: Person = ask("test", output_model=Person)
    assert result.dummy is True

    # 環境変数を変更
    os.environ["DARIKO_API_KEY"] = "direct_key"
    set_config(model="gpt-4o-mini", llm_key="direct_key")
    result: Person = ask("test", output_model=Person)
    assert result.dummy is True 



import requests
from .llm import LLM

class Claude(LLM):
    def __init__(self, model_name: str, llm_key: str):
        super().__init__(model_name, llm_key)
        self.api_url = "https://api.anthropic.com/v1/messages"

    def call(self, messages):
        if not self.llm_key:
            raise ValueError("APIキーが必要です")
        headers = {
            "x-api-key": self.llm_key,
            "anthropic-version": "2023-06-01",
            "content-type": "application/json"
        }
        # Claude APIのメッセージ形式に変換
        prompt = self._format_messages(messages)
        payload = {
            "model": self.model_name,
            "max_tokens": 1024,
            "messages": [{"role": "user", "content": prompt}]
        }
        resp = requests.post(self.api_url, headers=headers, json=payload)
        resp.raise_for_status()
        return resp.json()["content"][0]["text"]

    def _format_messages(self, messages):
        return "\n".join([f"{m['role']}: {m['content']}" for m in messages]) 



import requests
from typing import Dict, List

from .llm import LLM


class GPT(LLM):
    """OpenAIのGPTモデル用の実装"""

    def __init__(self, model_name: str, llm_key: str):
        super().__init__(model_name=model_name, llm_key=llm_key)
        self.api_url = "https://api.openai.com/v1/chat/completions"

    def call(self, messages: List[Dict[str, str]]) -> str:
        """OpenAI APIを呼び出して応答を取得する"""
        if not self.llm_key:
            raise ValueError("API key is required for OpenAI models")

        r = requests.post(
            self.api_url,
            headers={
                "Authorization": f"Bearer {self.llm_key}",
                "Content-Type": "application/json",
            },
            json={
                "model": self.model_name,
                "messages": messages,
                "response_format": {"type": "json_object"},
            },
            timeout=30,
        )

        if r.status_code != 200:
            raise RuntimeError(f"OpenAI API call failed: {r.text}")

        return r.json()["choices"][0]["message"]["content"]



from unittest.mock import patch

from dariko import ask, ask_batch, set_config
from tests.conftest import Person, mock_gpt_response


@patch("dariko.models.gpt.requests.post", side_effect=mock_gpt_response)
def test_ask_with_variable_annotation(mock_post):
    """変数アノテーションを使用したaskのテスト"""
    set_config(model="gpt-4o-mini", llm_key="test_key")
    result: Person = ask("test", output_model=Person)
    assert isinstance(result, Person)
    assert result.dummy is True


@patch("dariko.models.gpt.requests.post", side_effect=mock_gpt_response)
def test_ask_with_return_type(mock_post):
    """戻り値の型アノテーションを使用したaskのテスト"""
    set_config(model="gpt-4o-mini", llm_key="test_key")

    def get_person(prompt: str) -> Person:
        return ask(prompt, output_model=Person)

    result = get_person("test")
    assert isinstance(result, Person)
    assert result.dummy is True


@patch("dariko.models.gpt.requests.post", side_effect=mock_gpt_response)
def test_ask_with_explicit_model(mock_post):
    """明示的なモデル指定を使用したaskのテスト"""
    set_config(model="gpt-4o-mini", llm_key="test_key")
    result = ask("test", output_model=Person)
    assert isinstance(result, Person)
    assert result.dummy is True


@patch("dariko.models.gpt.requests.post", side_effect=mock_gpt_response)
def test_ask_batch(mock_post):
    """バッチ処理のテスト"""
    set_config(model="gpt-4o-mini", llm_key="test_key")
    prompts = ["test1", "test2"]
    results: list[Person] = ask_batch(prompts, output_model=Person)
    assert len(results) == 2
    assert all(isinstance(r, Person) for r in results)
    assert all(r.dummy is True for r in results) 



import os
from pydantic import BaseModel

from dariko import ask, set_config

# APIキーの設定(環境変数から取得)
llm_key = os.environ.get("DARIKO_API_KEY")
set_config(model="gpt-4o-mini", llm_key=llm_key)


class Person(BaseModel):
    name: str
    age: int
    dummy: bool


def get_person_info() -> Person:
    """人物情報を取得する関数"""
    return ask(
        '以下の形式のJSONを返してください:\n{"name": "山田太郎", "age": 25, "dummy": false}', output_model=Person
    )


# 関数の戻り値として使用
person = get_person_info()
print(f"名前: {person.name}")
print(f"年齢: {person.age}")
print(f"ダミー: {person.dummy}")



import os
from typing import Optional

from dotenv import load_dotenv

# .env ファイルを読み込む
load_dotenv()

# グローバル設定
_MODEL: str = "gpt-4o-mini"
_LLM_KEY: Optional[str] = None


def set_config(model: str, llm_key: Optional[str] = None) -> None:
    """
    モデルとLLMキー（APIキーまたはトークン）を設定する

    Args:
        model: 使用するLLMモデル名
        llm_key: APIキーまたはトークン（オプション）
    """
    global _MODEL, _LLM_KEY
    _MODEL = model
    _LLM_KEY = llm_key


def get_model() -> str:
    """設定されたモデル名を返す"""
    return _MODEL


def get_llm_key() -> Optional[str]:
    """設定されたLLMキーを返す"""
    return _LLM_KEY



from typing import Dict, List
import os
import torch
from transformers import AutoTokenizer, AutoModelForCausalLM

from .llm import LLM


class Gemma(LLM):
    """Google Gemmaモデル用の実装"""

    def __init__(self, model_name: str, llm_key: str = None):
        super().__init__(model_name=model_name, llm_key=llm_key)
        if not llm_key:
            raise ValueError("Hugging Face token is required for Gemma models")

        self.tokenizer = AutoTokenizer.from_pretrained(model_name, token=llm_key)
        self.model = AutoModelForCausalLM.from_pretrained(
            model_name, device_map="auto", torch_dtype=torch.float16, token=llm_key
        )

    def call(self, messages: List[Dict[str, str]]) -> str:
        """Gemmaモデルを呼び出して応答を取得する"""
        # メッセージをプロンプト形式に変換
        prompt = self._format_messages(messages)

        # トークン化
        inputs = self.tokenizer(prompt, return_tensors="pt").to(self.model.device)

        # 生成
        outputs = self.model.generate(**inputs, max_new_tokens=512, temperature=0.7, do_sample=True)

        # デコード
        response = self.tokenizer.decode(outputs[0], skip_special_tokens=True)

        # プロンプト部分を除去
        return response[len(prompt) :]

    def _format_messages(self, messages: List[Dict[str, str]]) -> str:
        """メッセージリストをプロンプト形式に変換"""
        formatted = ""
        for msg in messages:
            role = msg["role"]
            content = msg["content"]
            formatted += f"{role}: {content}\n"
        return formatted



from pydantic import ValidationError as _PydanticValidationError


class ValidationError(Exception):
    """LLM 出力の型検証エラーを表す例外"""

    def __init__(self, original: _PydanticValidationError):
        super().__init__(str(original))
        self.original = original



from unittest.mock import patch

from dariko import ask, set_config
from tests.conftest import Person, mock_gemma_response


@patch("dariko.models.gemma.Gemma.call", side_effect=mock_gemma_response)
@patch("transformers.AutoTokenizer.from_pretrained")
@patch("transformers.AutoModelForCausalLM.from_pretrained")
def test_configure_gemma(mock_model, mock_tokenizer, mock_call):
    """Gemmaモデルの設定テスト"""
    # Hugging Faceのトークンを設定
    set_config(model="google/gemma-2b", llm_key="test_hf_token")
    result: Person = ask("test", output_model=Person)
    assert result.dummy is True 
